﻿using System;
using System.Linq;
using System.Windows;
using System.Collections.ObjectModel;
using System.Collections;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;

namespace WpfApplication2
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = new SampleViewModel();

            this.treeView1.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery), true);
            this.treeView2.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery), true);
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            RadTreeViewItem destinationItem = e.Options.Destination as RadTreeViewItem;

            object source = this.GetItemFromPayload<object>(e.Options.Payload);
            object target = destinationItem != null ? destinationItem.Item : null;
            DropPosition position = destinationItem != null ? destinationItem.DropPosition : DropPosition.Inside;

            if (source != null && target != null)
            {
                Group sourceGroup = source as Group;
                Group targetGroup = target as Group;
                Person sourcePerson = source as Person;
                Person targetPerson = target as Person;

                if (sourceGroup != null)
                {
                    e.QueryResult = false;
                    return;
                }

                if (sourcePerson != null)
                {
                    if (sourcePerson != null && targetPerson != null && object.ReferenceEquals(sourcePerson, targetPerson))
                    {
                        e.QueryResult = false;
                        return;
                    }

                    if (targetPerson != null && position == DropPosition.Inside)
                    {
                        // do not allow dropping inside person
                        e.QueryResult = false;
                        return;
                    }

                    if (position != DropPosition.Inside && targetPerson == null)
                    {
                        e.QueryResult = false;
                        return;
                    }
                }
            }
            else
            {
                e.QueryResult = false;
                return;
            }
            e.QueryResult = true;
        }

        private T GetItemFromPayload<T>(object payload)
        {
            IEnumerable draggedItems = payload as IEnumerable;
            if (draggedItems != null)
            {
                return draggedItems.OfType<T>().FirstOrDefault();
            }
            return default(T);
        }
    }

    public class Person
    {
        public string Name { get; set; }

        public Person(string name)
        {
            this.Name = name;
        }
    }

    public class Group
    {
        public string Name { get; set; }
        public ObservableCollection<Person> Participants { get; private set; }

        public Group()
        {
            this.Participants = new ObservableCollection<Person>();
        }
    }

    public class SampleViewModel
    {
        public ObservableCollection<Group> GroupStaff { get; private set; }
        public ObservableCollection<Group> GroupTeams { get; private set; }

        public SampleViewModel()
        {
            this.GroupStaff = new ObservableCollection<Group>();
            this.GroupTeams = new ObservableCollection<Group>();

            Group gr = new Group();
            gr.Name = "QAS";
            this.LoadPeopleInGroup(gr, "Nancy Davolio", "Andrew Fuller", "Janet Levering", "Margaret Peacock", "Steven Buchanan");
            GroupStaff.Add(gr);

            gr = new Group();
            gr.Name = "Designers";
            this.LoadPeopleInGroup(gr, "Maria Anders", "Ana Trujillo", "Antonio Moreno", "Thomas Hardy", "Christina Berglund");
            GroupStaff.Add(gr);

            gr = new Group();
            gr.Name = "Sales";
            this.LoadPeopleInGroup(gr, "Hanna Moos", "Laurence Lebihan", "Elizabeth Lincoln", "Victoria Ashworth", "Patricio Simpson");
            GroupStaff.Add(gr);

            gr = new Group();
            gr.Name = "Developers";
            this.LoadPeopleInGroup(gr, "Francisco Chang", "Yang Wang", "Pedro Afonso", "Elizabeth Brown", "Sven Ottlieb");
            GroupStaff.Add(gr);

            gr = new Group();
            gr.Name = "Managers";
            this.LoadPeopleInGroup(gr, "Janine Labrune", "Ann Devon", "Roland Mendel", "Aria Cruz", "Diego Roel");
            GroupStaff.Add(gr);

            gr = new Group();
            gr.Name = "XamlTeam";
            GroupTeams.Add(gr);

            gr = new Group();
            gr.Name = "WebTeam";
            GroupTeams.Add(gr);

            gr = new Group();
            gr.Name = "AspNetTeam";
            GroupTeams.Add(gr);
        }

        private void LoadPeopleInGroup(Group group, string firstPerson, string secondPerson,
            string thirdPerson, string fourthPerson, string fifthPerson)
        {
            group.Participants.Add(new Person(firstPerson));
            group.Participants.Add(new Person(secondPerson));
            group.Participants.Add(new Person(thirdPerson));
            group.Participants.Add(new Person(fourthPerson));
            group.Participants.Add(new Person(fifthPerson));
        }
    }
}

